<?php
/**
 * @description: Main LabelCall class
 * @author: ENDORA
 * @website: https://endora.pl
 *
 * Main LabelCall class
 */

namespace labelcall;

require_once 'LabelCallApi.php';

if (!class_exists('LabelCall')) {
	class LabelCall {
		private static $instance = null;
		public $labelCall = null;
		public $masterAccountId = null;
		public $apiKey = null;

		public function __construct() {
			if ($this->get_merchant_properties()) {
				$this->labelCall = new LabelCallApi($this->masterAccountId, $this->apiKey);
				/*
				* Action hooks
				*/
				add_action('woocommerce_update_order', array($this, 'user_profile_update_by_order'));
				add_action('woocommerce_checkout_order_processed', array($this, 'user_profile_update_by_order'));
				add_action('woocommerce_customer_save_address', array($this, 'user_profile_updated'));
				add_action('profile_update', array($this, 'user_profile_updated'));
			}
		}

		/**
		 * Method check if merchant set up personal properties
		 *
		 * @return bool
		 */
		public function get_merchant_properties() {
			if (strlen(get_option('labelcall_master_account_id')) > 0 && strlen(get_option('labelcall_api_key')) > 0) {
				$this->masterAccountId = get_option('labelcall_master_account_id');
				$this->apiKey = get_option('labelcall_api_key');

				return true;
			} else {
				return false;
			}
		}

		/**
		 * Method that scrape all needed user data by order_id
		 *
		 * @param $order_id
		 */
		public function user_profile_update_by_order( $order_id) {
			error_log('order_updated\created');
			$last_order = wc_get_order($order_id);
			$this->user_profile_update_by_profile(null, $last_order);
		}

		public function user_profile_updated( $user_id) {
			$this->user_profile_update_by_profile($user_id);
		}

		/**
		 * Method that scrape all needed user data by user_id
		 *
		 * @param $user_id
		 * @param null $last_order
		 * @return array|bool
		 */
		public function user_profile_update_by_profile( $user_id, $last_order = null) {
			$result          = array();
			$data_object = null;

			if (!is_null($last_order) && $last_order->get_billing_phone()) {
				$customer_phone = $last_order->get_billing_phone(); 
			} else {
				$data_object = get_user_meta($user_id);

				if (isset($data_object['billing_phone'][0])) {
					$customer_phone = $data_object['billing_phone'][0];
				} else {
					$customer_phone = null;
				}
			}

			if ($customer_phone) {
				if ($last_order) {
					$data_object = $last_order;
				} else {
					if (is_null($data_object)) {
						$data_object = get_user_meta($user_id);
					}
				}
	
				$meta = $this->buildProps($customer_phone, $data_object);
				$result = $this->sendData($customer_phone, $meta);
			}

			return $result;
		}

		/**
		 * Singleton for class
		 *
		 * @return LabelCall|null
		 */
		public static function instance() {
			if (self::$instance) {
				return self::$instance;
			} else {
				self::$instance = new LabelCall();
				return self::$instance;
			}
		}

		public function removeIntegration() {
			if (get_option('labelcall_master_account_id')) {
				delete_option('labelcall_master_account_id');
			}

			if (get_option('labelcall_api_key')) {
				delete_option('labelcall_api_key');
			}
			
			if ($this->labelCall) {
				$this->labelCall->removeIntegration();
			}
		}

		private function buildProps( $phone, $data_object) {
			global $woocommerce;
			$cost = 0;
			$num_valid_orders = 0;
			$num_products_in_shopping_cart = 0;

			$orders_query = new \WC_Order_Query(array(
				'billing_phone' => $phone
			));

			$results = $orders_query->get_orders();

			if (count($results) > 0) {
				foreach ($results as $order) {
					if ('cancelled' != $order->get_status() && 'refunded' != $order->get_status()) {
						$cost += $order->get_total();
						$num_valid_orders++;
					}
				}
			}
	
			if ($data_object instanceof \WC_Order) {
				$user = $data_object->get_user();
				$name = $user->first_name . ' ' . $user->last_name;
				foreach ($data_object->get_items() as $item) {
					$num_products_in_shopping_cart += $item->get_quantity();
				}
			} else {
				$name = $data_object['first_name'][0] . ' ' . $data_object['last_name'][0];
				$num_products_in_shopping_cart = $woocommerce->cart->get_cart_contents_count();
			}

			$props = array();

			if ($name) {
				array_push($props, array(
					'PropertyId'     => 'cname',
					'Value'    => $name,
					'Type'     => LabelCallApi::PROPERTY_TYPE_NAME,
					'DeviceGroupId' => 'default'
				));
			}

			array_push($props, array(
				'PropertyId'     => 'wc_address-base',
				'Value'    => $this->getProp($data_object, 'billing_address_1'),
				'Type'     => LabelCallApi::PROPERTY_TYPE_LABEL,
				'DeviceGroupId' => 'default'
			));

			array_push($props, array(
				'PropertyId'     => 'wc_address-extra',
				'Value'    => $this->getProp($data_object, 'billing_address_2'),
				'Type'     => LabelCallApi::PROPERTY_TYPE_LABEL,
				'DeviceGroupId' => 'default'
			));

			if ($this->getProp($data_object, 'billing_company')) {
				array_push($props, array(
					'PropertyId'     => 'wc_company',
					'Value'    => $this->getProp($data_object, 'billing_company'),
					'Type'     => LabelCallApi::PROPERTY_TYPE_LABEL,
					'DeviceGroupId' => 'default'
				));
			}

			if ($num_valid_orders > 0) {
				array_push($props, array(
					'PropertyId'     => 'wc_mean-value-of-orders',
					'Value'    => round($cost / $num_valid_orders),
					'Type'     => LabelCallApi::PROPERTY_TYPE_DECIMAL,
					'DeviceGroupId' => 'default'
				));
			}

			if ($data_object instanceof \WC_Order) {
				array_push($props, array(
					'PropertyId'     => 'wc_last-order-id',
					'Value'    => $data_object->get_id(),
					'Type'     => LabelCallApi::PROPERTY_TYPE_NUMBER,
					'DeviceGroupId' => 'default'
				));
			}

			if ($data_object instanceof \WC_Order) {
				$notes = wc_get_order_notes(array('order_id' => $data_object->get_id(), 'order_by' => 'data_created'));

				if ($notes) {
					array_push($props, array(
						'PropertyId'     => 'wc_last-order-note',
						'Value'    => $notes[0]->content,
						'Type'     => LabelCallApi::PROPERTY_TYPE_DESCRIPTION,
						'DeviceGroupId' => 'default'
					));
				}
			}

			if ($data_object instanceof \WC_Order) {
				array_push($props, array(
					'PropertyId'     => 'wc_last-order-activity-date',
					'Value'    => $data_object->get_date_modified()->date('d-m-Y H:i:s'),
					'Type'     => LabelCallApi::PROPERTY_TYPE_DATE,
					'DeviceGroupId' => 'default'
				));
			}

			if ($data_object instanceof \WC_Order) {
				array_push($props, array(
					'PropertyId'     => 'wc_last-order-date',
					'Value'    => $data_object->get_date_created()->date('d-m-Y H:i:s'),
					'Type'     => LabelCallApi::PROPERTY_TYPE_DATE,
					'DeviceGroupId' => 'default'
				));
			}

			if ($data_object instanceof \WC_Order) {
				array_push($props, array(
					'PropertyId'     => 'wc_last-order-status',
					'Value'    => $data_object->get_status(),
					'Type'     => LabelCallApi::PROPERTY_TYPE_LABEL,
					'DeviceGroupId' => 'default'
				));
			}

			if ($data_object instanceof \WC_Order && $data_object->get_customer_note()) {
				array_push($props, array(
					'PropertyId'  => 'wc_last-order-user-comment',
					'Value' => $data_object->get_customer_note(),
					'Type'  => LabelCallApi::PROPERTY_TYPE_DESCRIPTION,
					'DeviceGroupId' => 'default'
				));
			}

			array_push($props, array(
				'PropertyId'     => 'wc_number-of-orders',
				'Value'    => $num_products_in_shopping_cart,
				'Type'     => LabelCallApi::PROPERTY_TYPE_NUMBER,
				'DeviceGroupId' => 'default'
			));

			if ($data_object instanceof \WC_Order) {
				$value_of_order = $data_object->get_subtotal();
			} else {
				$value_of_order = $woocommerce->cart->get_subtotal();
			}

			array_push($props, array(
				'PropertyId'     => 'wc_value-of-order',
				'Value'    => $value_of_order,
				'Type'     => LabelCallApi::PROPERTY_TYPE_NUMBER,
				'DeviceGroupId' => 'default'
			));

			array_push($props, array(
				'PropertyId'     => 'wc_value-of-orders',
				'Value'    => $cost,
				'Type'     => LabelCallApi::PROPERTY_TYPE_NUMBER,
				'DeviceGroupId' => 'default'
			));

			array_push($props, array(
				'PropertyId'     => 'wc_city',
				'Value'    => $this->getProp($data_object, 'billing_city'),
				'Type'     => LabelCallApi::PROPERTY_TYPE_LABEL,
				'DeviceGroupId' => 'default'
			));

			array_push($props, array(
				'PropertyId'     => 'wc_postcode',
				'Value'    => $this->getProp($data_object, 'billing_postcode'),
				'Type'     => LabelCallApi::PROPERTY_TYPE_LABEL,
				'DeviceGroupId' => 'default'
			));

			return $props;
		}

		private function sendData( $phone, $props) {
			$result = false;

			if (null != $phone && null != $this->masterAccountId && null != $this->apiKey) {
				if (strpos($phone, '+48') !== false) {
					$apiResult = $this->labelCall->upsertContact($phone, $props);
				} else {
					$apiResult = $this->labelCall->upsertContact('+48' . $phone, $props);
				}

				if ($apiResult['IsSuccess']) {
					$result = true;
				}
			}

			return $result;
		}

		private function getProp( $obj, $name) {
			$result = null;
			$methodName = 'get_' . $name;

			if (method_exists($obj, $methodName)) {
				$result = $obj->{$methodName}();
			} else {
				if (!$obj instanceof \WC_Order) {
					$result = $obj[$name][0];
				}
			}

			return $result;
		}
	}
}
